'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = session;

var _cspaceApi = require('cspace-api');

var _cspaceApi2 = _interopRequireDefault(_cspaceApi);

var _urlJoin = require('url-join');

var _urlJoin2 = _interopRequireDefault(_urlJoin);

var _tokenStore = require('./tokenStore');

var _tokenStore2 = _interopRequireDefault(_tokenStore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultSessionConfig = {
  username: '',
  password: ''
}; /* global btoa */

const base64Encode = value => {
  if (typeof value === 'undefined' || value === null) {
    return value;
  }

  if (typeof btoa !== 'undefined') {
    return btoa(value);
  }

  return new Buffer(value).toString('base64');
};

function session(sessionConfig) {
  const config = Object.assign({}, defaultSessionConfig, sessionConfig);
  const authStore = (0, _tokenStore2.default)(config.clientId, config.url);

  let authRequestPending = null;
  let auth = authStore.fetch() || {};

  if (config.username) {
    // The username for this session was specified, and differs from the stored auth user.
    // Don't use the stored auth.

    if (config.username !== auth.username) {
      auth = {};
    }
  } else if (auth.username) {
    // The username for this session was not specified. Use the stored auth user, if one exists.

    config.username = auth.username;
  }

  const cs = (0, _cspaceApi2.default)({
    url: (0, _urlJoin2.default)(config.url, 'cspace-services')
  });

  const csAuth = (0, _cspaceApi2.default)({
    url: (0, _urlJoin2.default)(config.url, 'cspace-services/oauth'),
    username: config.clientId,
    password: config.clientSecret,
    type: 'application/x-www-form-urlencoded'
  });

  const storeToken = response => {
    auth = {
      username: config.username,
      accessToken: response.data.access_token,
      refreshToken: response.data.refresh_token
    };

    authStore.store(auth);

    // We have tokens, so the user password can be discarded.

    delete config.password;

    return response;
  };

  const authRequest = data => {
    if (authRequestPending) {
      return authRequestPending;
    }

    authRequestPending = csAuth.create('token', { data }).then(response => storeToken(response)).then(response => {
      authRequestPending = null;

      return response;
    });

    return authRequestPending;
  };

  const login = () => authRequest({
    grant_type: 'password',
    username: config.username,
    password: base64Encode(config.password)
  });

  const refresh = () => authRequest({
    grant_type: 'refresh_token',
    refresh_token: auth.refreshToken
  });

  const logout = () => new Promise(resolve => {
    // Log out may in the future require an async call to the REST API (for example, to revoke
    // tokens immediately). Currently it's a client-side only operation that can be done
    // synchronously, but to be consistent with a future async operation, we'll simulate it with
    // setTimeout.

    setTimeout(() => {
      delete config.username;
      delete config.password;

      auth = {};
      authStore.clear();

      resolve({});
    });
  });

  const tokenizeRequest = requestConfig => Object.assign({}, requestConfig, { token: auth.accessToken });

  const tokenized = operation => (resource, requestConfig) => cs[operation](resource, tokenizeRequest(requestConfig)).catch(error => {
    if (error.response && error.response.status === 401 && auth.refreshToken) {
      // Refresh the access token and retry.

      return refresh().then(() => cs[operation](resource, tokenizeRequest(requestConfig)));
    }

    return Promise.reject(error);
  }).catch(error => {
    // Invoke the configured error handler, if any.

    if (config.onError) {
      return config.onError(error);
    }

    return Promise.reject(error);
  });

  return {
    config() {
      const configCopy = Object.assign({}, config);

      delete configCopy.clientSecret;
      delete configCopy.password;

      return configCopy;
    },
    login,
    logout,
    create: tokenized('create'),
    read: tokenized('read'),
    update: tokenized('update'),
    delete: tokenized('delete')
  };
}